#include "mt64.h"
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char ** argv)
{
   //+------------------+
   //|----              |
   //|    ----          |
   //|        ---       |
   //|           ---    |
   //|              -   |
   //|               |  |
   //|                | |
   //|                 ||
   //+------------------+

   //Based on wolfram.com
   //Assume we have a square.
   //If this square has sides of length 2, its area is 2*2 = 4
   //Let's put a circle inside the square and assume the radius of the
   //circle is zero.
   //Let's also make the center of the square the same as the center of 
   //the circle, so the square circumscribes (contains) the circle.
   //Now the area of the circle is PI.
   //The difference between the two is 4 - pi.
   //
   //Notice that the proportion of the area of the circle to the area 
   //of the rectangle is PI/4.  If we use a large number of dots and
   //place them with uniform distribution inside the square, some will
   //fall within the circle and others will fall outside the circle, but 
   //within the square.  Notice that the proportion is the same as the 
   //proportion of the areas, but we will have to use more than one 
   //set of values to deal with this:
   //(1 >= x >= 0, 1 >= y >= 0 )
   //(0 >= x >= -1, 1 >= y >= 0 )
   //(1 >= x >= 0, 0 >= y >= -1 )
   //(0 >= x >= -1, 0 >= y >= -1 )
   //
   //We can actually use only 1/4 of the square and circle with the same
   //proportion

   unsigned long long init[4]={0x12345ULL, 0x23456ULL, 0x34567ULL, 0x45678ULL}, length=4;
   init_by_array64(init, length);

   long count = 0L;
   long i = 0L;
   double x, y;

   for(i = 0L; i < 1000000000L; i++)
   {
     x = genrand64_real1();
     y = genrand64_real1();
     if(x*x + y*y <= 1.0)
        count++;
   }

   printf("%20.19lf\n", 4.0*((double)count)/(double)i );
}

